import os, uuid
from django.utils.text import slugify
from django.db import models


def get_upload_path(instance, filename):
    """Returns a unique path up to 100 characters.
    """
    name, ext = os.path.splitext(filename)
    path = os.path.join(
        instance._meta.object_name.lower(),
        uuid.uuid4().hex,
        slugify(name),
    )
    return f'{path[:90]}{ext[:10]}'


class RedactorFile(models.Model):
    file = models.FileField(
        upload_to=get_upload_path,
    )

    name = models.CharField(max_length=255)

    is_image = models.BooleanField()

    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['-id']



from io import BytesIO

from django.conf import settings
from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.files.uploadedfile import SimpleUploadedFile
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from PIL import Image, UnidentifiedImageError

from .models import RedactorFile


@method_decorator(csrf_exempt, name='dispatch')
class RedactorView(UserPassesTestMixin, View):
    def test_func(self):
        return self.request.user.is_authenticated and self.request.user.is_staff

    def get(self, request, *args, **kwargs):
        return JsonResponse(
            [
                {
                    'thumb': obj.file.url, 'url': obj.file.url, 'id': obj.id, 'title': obj.name
                } for obj in RedactorFile.objects.filter(is_image=True)[:20]
            ],
            safe=False
        )

    def post(self, request, *args, **kwargs):
        count = 0
        images = {}
        for file in request.FILES.getlist('file[]'):
            if getattr(settings, 'AWS_ACCESS_KEY_ID', None):
                obj = RedactorFile.objects.create(file=file, name=file.name, is_image=False)

            else:
                try:
                    output_image = Image.open(file)
                    img_format = output_image.format
                    if output_image.format == 'JPEG':
                        output_image = output_image.convert('RGB')

                    bytes_io = BytesIO()
                    output_image.save(bytes_io, format=img_format, optimize=True)

                    obj = RedactorFile.objects.create(
                        file=SimpleUploadedFile(file.name, bytes_io.getvalue()),
                        name=file.name,
                        is_image=True,
                    )
                except UnidentifiedImageError:
                    obj = RedactorFile.objects.create(file=file, name=file.name, is_image=False)

            images[f'file-{count}'] = {'url': obj.file.url, 'name': obj.name, 'id': obj.id}
            count += 1

        return JsonResponse(images, safe=False)



class RedactorMixin:
    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)

        for field in getattr(self, 'redactor_fields', []):
            form.base_fields[field].widget.attrs['enable-redactor'] = True

        for field in getattr(self, 'basic_redactor_fields', []):
            form.base_fields[field].widget.attrs['enable-basic-redactor'] = True

        return form

    class Media:
        css = {
            'all': [
                'vendor/redactor/redactor.min.css',
                'redactor/admin-redactor.css',
            ]
        }
        js = [
            'vendor/redactor/redactor.min.js',
            'vendor/redactor/plugins/imagemanager.min.js',
            'vendor/redactor/plugins/video.min.js',
            'vendor/redactor/plugins/widget.min.js',
            'redactor/admin-redactor.js',
        ]



# Generated by Django 2.2.6 on 2019-11-26 14:32

import redactor.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RedactorFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to=redactor.models.get_upload_path)),
                ('name', models.CharField(max_length=255)),
                ('is_image', models.BooleanField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['-id'],
            },
        ),
    ]
